@@ -192,6 +192,11 @@ restructure_v2 <- function(x) {
     # added support for OpenCL
     x$opencl <- opencl()
   }
+  if (version < "2.16.1") {
+    # problems with rstan::read_stan_csv as well as
+    # non-unique variable names became apparent (#1218)
+    x$fit <- repair_stanfit_names(x$fit)
+  }
   x
 }
 
